﻿function MapHelper(options) {
    var self = this;
    //  Initialize with the options supplied
    this.mapSelector = options.mapSelector;
    this.shapeDataSource = options.shapeDataSource;

    // Attaches an event handler which zooms the map so that all map shapes are visible
    this.autoZoom = function() {
        $(document).on("igmaprefreshcompleted", self.mapSelector, autoZoomHandler);
    }

    //  Finds the extent of all shapes in the map control and calculates a proper
    //  map window so that all shapes are visible.
    //  Detaches from the refreshCompleted event of the map control after 
    //  themap rectangle is set to the map control
    function autoZoomHandler() {
        var shapes = self.getAllShapesExtent();
        if (shapes.length > 0) {
            var allShapesBounds = self.findShapeArrayBounds(shapes);
            var mapWindow = self.calculateMapWindow(allShapesBounds);
            $(self.mapSelector).igMap("option", "windowRect", mapWindow);
            $(document).off("igmaprefreshcompleted", self.mapSelector, autoZoomHandler);
        }
    }

    this.mapShapes = function (mapShape) {
        var shapeData = self.shapeDataSource;
        var shapeEnumerator = shapeData.converter().getEnumerator();
        var mappedShapes = [];
        while (shapeEnumerator.moveNext()) {
            mappedShapes.push(mapShape(shapeEnumerator.current()));
        }

        return mappedShapes;
    }

    this.getAllShapesExtent = function () {
        var shapeData = self.shapeDataSource;
        var shapeEnumerator = shapeData.converter().getEnumerator();
        var shapesArray = [];
        while (shapeEnumerator.moveNext()) {
            shapesArray.push(shapeEnumerator.current());
        }
        return shapesArray;
    }

    this.findShapeArrayBounds = function(shapeArray) {
        //  Store to improve performance and readability
        var sCount = shapeArray.length;

        if (sCount > 0) {
            var left, top, right, bottom;

            //  Enumerate shapes
            for (var s = 0; s < sCount; s++) {
                var currentShapeBounds = self.findShapeBounds(shapeArray[s]);

                if (currentShapeBounds.left < left || !left) left = currentShapeBounds.left;
                if (currentShapeBounds.right > right || !right) right = currentShapeBounds.right;

                if (currentShapeBounds.top > top || !top) top = currentShapeBounds.top;
                if (currentShapeBounds.bottom < bottom || !bottom) bottom = currentShapeBounds.bottom;
            }

            return {
                left: left,
                right: right,
                top: top,
                bottom: bottom
            };
        }
    }

    this.findShapeBounds = function(shape) {
        var left, top, right, bottom;
        var points = shape.points.item(0);
        var pCount = points.count();
        //  Enumerate shape points
        if (pCount > 0) {
            //  Find bounds of the state
            for (var i = 0; i < pCount; i++) {
                currentPoint = points.item(i);

                if (currentPoint.__x < left || !left) left = currentPoint.__x;
                if (currentPoint.__x > right || !right) right = currentPoint.__x;

                if (currentPoint.__y > top || !top) top = currentPoint.__y;
                if (currentPoint.__y < bottom || !bottom) bottom = currentPoint.__y;
            }

            return {
                left: left,
                right: right,
                top: top,
                bottom: bottom
            };
        }
    }

    this.calculateMapWindow = function(minViewWindow, zoomRatio) {
        if (!zoomRatio) {
            zoomRatio = 1;
        }
        //  Calculate central point and required radius
        var width = minViewWindow.right - minViewWindow.left;
        var height = minViewWindow.top - minViewWindow.bottom;
        var centered = {
            longitude: minViewWindow.right - width / 2,
            latitude: minViewWindow.top - height / 2,
            radius: (width > height) ? width / 2 * zoomRatio : height / 2 * zoomRatio
        };
        //  Calculate map window in relative units
        var zoomRect = $(self.mapSelector).igMap("getZoomFromGeographic", self.geographicFromCentered(centered));
        return zoomRect;
    }

    //  Calculates the geographic coordinates of a square around a central point and radius
    this.geographicFromCentered = function(centered) {
        var geographic =
        {
            left: centered.longitude - centered.radius,
            top: centered.latitude - centered.radius,
            width: centered.radius * 2,
            height: centered.radius * 2
        };
        return geographic;
    }
}